%Set tuning parameters for MH chain
burn_in=5*10^3;
chain_length=10^6+burn_in;
tuning=0.2;
rng(1)

noise_factor=0.1;

%create check function
rho=@(tau_temp,u) tau_temp-(u<=0);

%create moment function
gmat=@(tau_temp,beta,alpha) repmat(rho(tau_temp,y-d*beta-x*alpha),1,size(x,2)+size(z,2)).*[x z];
g=@(tau_temp,beta,alpha) mean(gmat(tau_temp,beta,alpha));

%create variance estimator
Sigma=@(tau_temp,beta,alpha) cov(gmat(tau_temp,beta,alpha));

%Define quasi-likelihood multiplied by prior.  Note that dropping constant
%terms since cancel out
logquasilikelihood=@(theta) -0.5*obs*g(tau,theta(1),theta(2:end,1))...
    *Sigma(tau,theta(1),theta(2:end,1))^-1*g(tau,theta(1),theta(2:end,1))';

%Define S statistic
S=@(theta) obs*g(tau,theta(1),theta(2:end,1))...
     *Sigma(tau,theta(1),theta(2:end,1))^-1*g(tau,theta(1),theta(2:end,1))';

%Calculate regression estimates to use as starting values
W=[d x];
%Choose starting value equal to OLS coefficient
theta_init=(W'*W)^-1*W'*y;
curr_log_obj=logquasilikelihood(theta_init);
curr_theta=theta_init;
theta_store=zeros(length(theta_init),chain_length);
accept_store=NaN(chain_length,1);
S_store=NaN(chain_length,1);
g_store=NaN(moment_dim,chain_length);
for n=1:chain_length
    trial_theta=curr_theta+randn(length(theta_init),1)*tuning;
    trial_log_obj=logquasilikelihood(trial_theta);
    if exp(trial_log_obj-curr_log_obj)>rand ...
                && trial_theta(1)>=beta_bounds(1) && trial_theta(1)<=beta_bounds(2)...
                && trial_theta(2)>=alpha_bounds(1) && trial_theta(2)<=alpha_bounds(2)
        curr_log_obj=trial_log_obj;
        curr_theta=trial_theta;
        accept_store(n)=1;
    else
        accept_store(n)=0;
    end
    theta_store(:,n)=curr_theta;
    %Store S statistic
    g_store(:,n)=Sigma(tau,curr_theta(1),curr_theta(2:end,1))^-.5*g(tau,curr_theta(1),curr_theta(2:end,1))';
    S_store(n)=-2*curr_log_obj;
end

theta_store(:,1:burn_in)=[];
S_store(1:burn_in)=[];

post_mean=mean(theta_store,2);
post_var=var(theta_store');

%Extract CU GMM estimate as best objective value visited
Max_location=S_store==min(S_store)';
GMM=theta_store(:,Max_location)';
GMM=GMM(1,:);
%First element corresponds to slope beta, second to intercept alpha

sigma_y=std(y)*noise_factor;

%create check function
rho=@(tau,u) tau-normcdf(-u/sigma_y);

%create moment function
gmat=@(tau,beta,alpha) repmat(rho(tau,y-d*beta-x*alpha),1,size(x,2)+size(z,2)).*[x z];
g=@(tau,beta,alpha) mean(gmat(tau,beta,alpha));


weights=@(gamma) exp(gmat(tau,GMM(1),GMM(2))*gamma)/sum(exp(gmat(tau,GMM(1),GMM(2))*gamma));
weighted_mean=@(gamma) mean(repmat(weights(gamma),1,moment_dim).*gmat(tau,GMM(1),GMM(2)));
weight_obj=@(gamma) 10^6*weighted_mean(gamma)*weighted_mean(gamma)';
gamma_opt=fminsearch(weight_obj,zeros(moment_dim,1));

sampling_weights=weights(gamma_opt);



